@@ -96,6 +96,20 @@ impl<'a, C: fmt::Debug + Sync + Send> fmt::Debug for Imp<'a, C> {
 }
 
 impl<'a> File<'a, ()> {
+    /// Wraps a [`fs::File`].
+    ///
+    /// The given `path` should be the path that has been used to
+    /// obtain `file` from.  It is used in error messages to provide
+    /// context to the user.
+    ///
+    /// While this is slightly less convenient than [`Self::open`], it
+    /// allows one to inspect or manipulate the [`fs::File`] object
+    /// before handing it off.  For example, one can inspect the
+    /// metadata.
+    pub fn new<P: AsRef<Path>>(file: fs::File, path: P) -> io::Result<Self> {
+        Self::new_with_cookie(file, path.as_ref(), ())
+    }
+
     /// Opens the given file.
     pub fn open<P: AsRef<Path>>(path: P) -> io::Result<Self> {
         Self::with_cookie(path, ())
@@ -103,8 +117,18 @@ impl<'a> File<'a, ()> {
 }
 
 impl<'a, C: fmt::Debug + Sync + Send> File<'a, C> {
-    /// Like `open()`, but sets a cookie.
-    pub fn with_cookie<P: AsRef<Path>>(path: P, cookie: C) -> io::Result<Self> {
+    /// Like [`Self::new`], but sets a cookie.
+    ///
+    /// The given `path` should be the path that has been used to
+    /// obtain `file` from.  It is used in error messages to provide
+    /// context to the user.
+    ///
+    /// While this is slightly less convenient than
+    /// [`Self::with_cookie`], it allows one to inspect or manipulate
+    /// the [`fs::File`] object before handing it off.  For example,
+    /// one can inspect the metadata.
+    pub fn new_with_cookie<P: AsRef<Path>>(file: fs::File, path: P, cookie: C)
+                                           -> io::Result<Self> {
         let path = path.as_ref();
 
         // As fallback, we use a generic reader.
@@ -115,8 +139,6 @@ impl<'a, C: fmt::Debug + Sync + Send> File<'a, C> {
                 path.into()))
         };
 
-        let file = fs::File::open(path).map_err(|e| FileError::new(path, e))?;
-
         // For testing and benchmarking purposes, we use the variable
         // SEQUOIA_DONT_MMAP to turn off mmapping.
         if ::std::env::var_os("SEQUOIA_DONT_MMAP").is_some() {
@@ -133,10 +155,10 @@ impl<'a, C: fmt::Debug + Sync + Send> File<'a, C> {
         }
 
         // Be nice to 32 bit systems.
-        if length > usize::max_value() as u64 {
-            return generic(file, cookie);
-        }
-        let length = length as usize;
+        let length: usize = match length.try_into() {
+            Ok(v) => v,
+            Err(_) => return generic(file, cookie),
+        };
 
         let fd = file.as_raw_fd();
         let addr = unsafe {
@@ -158,6 +180,13 @@ impl<'a, C: fmt::Debug + Sync + Send> File<'a, C> {
             path.into(),
         ))
     }
+
+    /// Like [`Self::open`], but sets a cookie.
+    pub fn with_cookie<P: AsRef<Path>>(path: P, cookie: C) -> io::Result<Self> {
+        let path = path.as_ref();
+        let file = fs::File::open(path).map_err(|e| FileError::new(path, e))?;
+        Self::new_with_cookie(file, path, cookie)
+    }
 }
 
 impl<'a, C: fmt::Debug + Sync + Send> io::Read for File<'a, C> {
